"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.AbstractAmqpConsumer = void 0;
const core_1 = require("@message-queue-toolkit/core");
const AbstractAmqpService_1 = require("./AbstractAmqpService");
const amqpMessageReader_1 = require("./amqpMessageReader");
const ABORT_EARLY_EITHER = { error: 'abort' };
class AbstractAmqpConsumer extends AbstractAmqpService_1.AbstractAmqpService {
    transactionObservabilityManager;
    errorResolver;
    executionContext;
    deadLetterQueueOptions;
    messageSchemaContainer;
    handlerContainer;
    constructor(dependencies, options, executionContext) {
        super(dependencies, options);
        this.transactionObservabilityManager = dependencies.transactionObservabilityManager;
        this.errorResolver = dependencies.consumerErrorResolver;
        this.deadLetterQueueOptions = options.deadLetterQueue;
        const messageSchemas = options.handlers.map((entry) => entry.schema);
        this.messageSchemaContainer = new core_1.MessageSchemaContainer({
            messageSchemas,
            messageTypeField: options.messageTypeField,
        });
        this.handlerContainer = new core_1.HandlerContainer({
            messageTypeField: this.messageTypeField,
            messageHandlers: options.handlers,
        });
        this.executionContext = executionContext;
    }
    async start() {
        await this.init();
        if (!this.channel)
            throw new Error('Channel is not set');
        await this.consume();
    }
    async init() {
        if (this.deadLetterQueueOptions) {
            // TODO: https://www.cloudamqp.com/blog/when-and-how-to-use-the-rabbitmq-dead-letter-exchange.html
            throw new Error('deadLetterQueue parameter is not currently supported by the Amqp adapter');
        }
        await super.init();
    }
    async receiveNewConnection(connection) {
        await super.receiveNewConnection(connection);
        await this.consume();
    }
    async consume() {
        await this.channel.consume(this.queueName, (message) => {
            if (message === null) {
                return;
            }
            const deserializedMessage = this.deserializeMessage(message);
            if (deserializedMessage.error === 'abort') {
                this.channel.nack(message, false, false);
                const messageId = this.tryToExtractId(message);
                this.handleMessageProcessed(null, 'invalid_message', messageId.result);
                return;
            }
            // @ts-ignore
            const messageType = deserializedMessage.result[this.messageTypeField];
            const transactionSpanId = `queue_${this.queueName}:${
            // @ts-ignore
            // eslint-disable-next-line @typescript-eslint/restrict-template-expressions
            deserializedMessage.result[this.messageTypeField]}`;
            this.transactionObservabilityManager?.start(transactionSpanId);
            if (this.logMessages) {
                const resolvedLogMessage = this.resolveMessageLog(deserializedMessage.result, messageType);
                this.logMessage(resolvedLogMessage);
            }
            this.internalProcessMessage(deserializedMessage.result, messageType)
                .then((result) => {
                if (result.error === 'retryLater') {
                    this.channel.nack(message, false, true);
                    this.handleMessageProcessed(deserializedMessage.result, 'retryLater');
                }
                if (result.result === 'success') {
                    this.channel.ack(message);
                    this.handleMessageProcessed(deserializedMessage.result, 'consumed');
                }
            })
                .catch((err) => {
                // ToDo we need sanity check to stop trying at some point, perhaps some kind of Redis counter
                // If we fail due to unknown reason, let's retry
                this.channel.nack(message, false, true);
                this.handleMessageProcessed(deserializedMessage.result, 'retryLater');
                this.handleError(err);
            })
                .finally(() => {
                this.transactionObservabilityManager?.stop(transactionSpanId);
            });
        });
    }
    async internalProcessMessage(message, messageType) {
        const prehandlerOutput = await this.processPrehandlers(message, messageType);
        const barrierResult = await this.preHandlerBarrier(message, messageType, prehandlerOutput);
        if (barrierResult.isPassing) {
            return this.processMessage(message, messageType, {
                prehandlerOutput,
                barrierOutput: barrierResult.output,
            });
        }
        return { error: 'retryLater' };
    }
    async processMessage(message, messageType, 
    // eslint-disable-next-line @typescript-eslint/no-explicit-any
    prehandlingOutputs) {
        const handler = this.handlerContainer.resolveHandler(messageType);
        return handler.handler(message, this.executionContext, prehandlingOutputs);
    }
    processPrehandlers(message, messageType) {
        const handlerConfig = this.handlerContainer.resolveHandler(messageType);
        return this.processPrehandlersInternal(handlerConfig.prehandlers, message);
    }
    async preHandlerBarrier(message, messageType, prehandlerOutput) {
        const handler = this.handlerContainer.resolveHandler(messageType);
        return this.preHandlerBarrierInternal(handler.preHandlerBarrier, message, this.executionContext, prehandlerOutput);
    }
    resolveSchema(message) {
        return this.messageSchemaContainer.resolveSchema(message);
    }
    resolveMessageLog(message, messageType) {
        const handler = this.handlerContainer.resolveHandler(messageType);
        return handler.messageLogFormatter(message);
    }
    // eslint-disable-next-line max-params
    resolveNextFunction(prehandlers, message, index, prehandlerOutput, resolve, reject) {
        return this.resolveNextPreHandlerFunctionInternal(prehandlers, this.executionContext, message, index, prehandlerOutput, resolve, reject);
    }
    deserializeMessage(message) {
        const resolveMessageResult = this.resolveMessage(message);
        if ((0, core_1.isMessageError)(resolveMessageResult.error)) {
            this.handleError(resolveMessageResult.error);
            return ABORT_EARLY_EITHER;
        }
        // Empty content for whatever reason
        if (!resolveMessageResult.result) {
            return ABORT_EARLY_EITHER;
        }
        const resolveSchemaResult = this.resolveSchema(resolveMessageResult.result);
        if (resolveSchemaResult.error) {
            this.handleError(resolveSchemaResult.error);
            return ABORT_EARLY_EITHER;
        }
        const deserializationResult = (0, core_1.parseMessage)(resolveMessageResult.result, resolveSchemaResult.result, this.errorResolver);
        if ((0, core_1.isMessageError)(deserializationResult.error)) {
            this.handleError(deserializationResult.error);
            return ABORT_EARLY_EITHER;
        }
        // Empty content for whatever reason
        if (!deserializationResult.result) {
            return ABORT_EARLY_EITHER;
        }
        return {
            result: deserializationResult.result,
        };
    }
    tryToExtractId(message) {
        const resolveMessageResult = this.resolveMessage(message);
        if ((0, core_1.isMessageError)(resolveMessageResult.error)) {
            this.handleError(resolveMessageResult.error);
            return ABORT_EARLY_EITHER;
        }
        // Empty content for whatever reason
        if (!resolveMessageResult.result) {
            return ABORT_EARLY_EITHER;
        }
        // @ts-ignore
        if (this.messageIdField in resolveMessageResult.result) {
            return {
                // @ts-ignore
                result: resolveMessageResult.result[this.messageIdField],
            };
        }
        return ABORT_EARLY_EITHER;
    }
    resolveMessage(message) {
        return (0, amqpMessageReader_1.readAmqpMessage)(message, this.errorResolver);
    }
}
exports.AbstractAmqpConsumer = AbstractAmqpConsumer;
//# sourceMappingURL=AbstractAmqpConsumer.js.map