"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.AbstractAmqpService = void 0;
const core_1 = require("@message-queue-toolkit/core");
const amqpInitter_1 = require("./utils/amqpInitter");
class AbstractAmqpService extends core_1.AbstractQueueService {
    connection;
    connectionManager;
    // @ts-ignore
    channel;
    isShuttingDown;
    queueName;
    constructor(dependencies, options) {
        super(dependencies, options);
        this.queueName = options.locatorConfig
            ? options.locatorConfig.queueName
            : options.creationConfig?.queueName;
        this.isShuttingDown = false;
        this.connectionManager = dependencies.amqpConnectionManager;
        this.connection = this.connectionManager.getConnectionSync();
        this.connectionManager.subscribeConnectionReceiver(this);
    }
    async receiveNewConnection(connection) {
        this.connection = connection;
        this.isShuttingDown = false;
        // If channel already exists, recreate it
        const oldChannel = this.channel;
        try {
            this.channel = await this.connection.createChannel();
        }
        catch (err) {
            // @ts-ignore
            this.logger.error(`Error creating channel: ${err.message}`);
            await this.connectionManager.reconnect();
            return;
        }
        if (oldChannel) {
            this.isShuttingDown = true;
            try {
                await oldChannel.close();
            }
            catch {
                // errors are ok
            }
            this.isShuttingDown = false;
        }
        if (this.deletionConfig && this.creationConfig) {
            await (0, amqpInitter_1.deleteAmqp)(this.channel, this.deletionConfig, this.creationConfig);
        }
        this.channel.on('close', () => {
            if (!this.isShuttingDown) {
                this.logger.error(`AMQP connection lost!`);
                this.reconnect().catch((err) => {
                    this.handleError(err);
                    throw err;
                });
            }
        });
        this.channel.on('error', (err) => {
            this.handleError(err);
        });
        if (this.creationConfig) {
            await this.channel.assertQueue(this.creationConfig.queueName, this.creationConfig.queueOptions);
        }
        else {
            await this.checkQueueExists();
        }
    }
    async checkQueueExists() {
        // queue check breaks channel if not successful
        const checkChannel = await this.connection.createChannel();
        checkChannel.on('error', () => {
            // it's OK
        });
        try {
            await checkChannel.checkQueue(this.locatorConfig.queueName);
            await checkChannel.close();
        }
        catch (err) {
            throw new Error(`Queue with queueName ${this.locatorConfig.queueName} does not exist.`);
        }
    }
    async destroyChannel() {
        if (this.channel) {
            try {
                await this.channel.close();
            }
            catch (err) {
                // We don't care about connection closing errors
            }
            finally {
                // @ts-ignore
                this.channel = undefined;
            }
        }
    }
    async init() {
        if (this.creationConfig?.updateAttributesIfExists) {
            throw new Error('updateAttributesIfExists parameter is not currently supported by the Amqp adapter');
        }
        // if we don't have connection yet, it's fine, we'll wait for a later receiveNewConnection() call
        if (this.connection) {
            await this.receiveNewConnection(this.connection);
        }
    }
    async reconnect() {
        await this.connectionManager.reconnect();
    }
    async close() {
        this.isShuttingDown = true;
        await this.destroyChannel();
    }
}
exports.AbstractAmqpService = AbstractAmqpService;
//# sourceMappingURL=AbstractAmqpService.js.map